use super::{Block as Blocked, UndoBlock};
use crate::{Completed, Context, Error, Required};
use activitystreams::{
    activity::{Block, Undo},
    base::AnyBase,
    context,
    prelude::*,
    security,
};

impl Completed for Blocked {
    fn to_apub(&self, ctx: &Context) -> Result<AnyBase, Error> {
        let block = ctx.store.view.blocks.by_id(self.block_id)?.req()?;
        let actor_id = ctx.apub.apub_for_profile(block.right)?.req()?;
        let object_id = ctx.apub.apub_for_profile(block.left)?.req()?;
        let published = block.published;

        let mut block = Block::new(actor_id, object_id);
        block
            .set_id(ctx.apub.info.gen_id())
            .set_published(published.into())
            .add_context(context())
            .add_context(security());
        let block = block.into_any_base()?;
        ctx.apub.store_object(&block)?;

        Ok(block)
    }
}

impl Completed for UndoBlock {
    fn to_apub(&self, ctx: &Context) -> Result<AnyBase, Error> {
        let person_id = ctx.apub.apub_for_profile(self.profile_id)?.req()?;
        let block = ctx.apub.object(&self.block_apub_id)?.req()?;

        let mut undo = Undo::new(person_id, block);
        undo.set_id(ctx.apub.info.gen_id())
            .add_context(context())
            .add_context(security());
        let undo = undo.into_any_base()?;
        ctx.apub.store_object(&undo)?;

        Ok(undo)
    }
}
